@@ -3,8 +3,8 @@
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
  * Copyright (C) 2017-2023 Osimis S.A., Belgium
- * Copyright (C) 2024-2024 Orthanc Team SRL, Belgium
- * Copyright (C) 2021-2024 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
+ * Copyright (C) 2024-2025 Orthanc Team SRL, Belgium
+ * Copyright (C) 2021-2025 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Affero General Public License
@@ -37,9 +37,10 @@
 #include <boost/algorithm/string/predicate.hpp>
 
 
+// we use "ResponseContent" in tools/find -> we need 1.12.5
 #define ORTHANC_CORE_MINIMAL_MAJOR     1
-#define ORTHANC_CORE_MINIMAL_MINOR     11
-#define ORTHANC_CORE_MINIMAL_REVISION  0
+#define ORTHANC_CORE_MINIMAL_MINOR     12
+#define ORTHANC_CORE_MINIMAL_REVISION  5
 
 static const char* const HAS_DELETE = "HasDelete";
 static const char* const SYSTEM_CAPABILITIES = "Capabilities";
@@ -486,7 +487,7 @@ static OrthancPluginErrorCode OnChangeCallback(OrthancPluginChangeType changeTyp
           if (hasExtendedFind)
           {
             LOG(INFO) << "Orthanc supports ExtendedFind.";
-            SetPluginCanUseExtendedFile(true);
+            SetPluginCanUseExtendedFind(true);
           }
           else
           {
@@ -527,6 +528,38 @@ static OrthancPluginErrorCode OnChangeCallback(OrthancPluginChangeType changeTyp
   return OrthancPluginErrorCode_Success;
 }
 
+template <bool isThumbnail>
+void RetrieveInstanceRenderedThumbnail(OrthancPluginRestOutput* output,
+                                       const char* url,
+                                       const OrthancPluginHttpRequest* request)
+{
+  RetrieveInstanceRendered(output, url, request, isThumbnail);
+}
+
+template <bool isThumbnail>
+void RetrieveFrameRenderedThumbnail(OrthancPluginRestOutput* output,
+                                    const char* url,
+                                    const OrthancPluginHttpRequest* request)
+{
+  RetrieveFrameRendered(output, url, request, isThumbnail);
+}
+
+template <bool isThumbnail>
+void RetrieveSeriesRenderedThumbnail(OrthancPluginRestOutput* output,
+                                     const char* url,
+                                     const OrthancPluginHttpRequest* request)
+{
+  RetrieveSeriesRendered(output, url, request, isThumbnail);
+}
+
+template <bool isThumbnail>
+void RetrieveStudyRenderedThumbnail(OrthancPluginRestOutput* output,
+                                    const char* url,
+                                    const OrthancPluginHttpRequest* request)
+{
+  RetrieveStudyRendered(output, url, request, isThumbnail);
+}
+
 
 
 extern "C"
@@ -661,10 +694,15 @@ extern "C"
 
         OrthancPlugins::RegisterRestCallback<GetClientInformation>(root + "info", true);
 
-        OrthancPlugins::RegisterRestCallback<RetrieveStudyRendered>(root + "studies/([^/]*)/rendered", true);
-        OrthancPlugins::RegisterRestCallback<RetrieveSeriesRendered>(root + "studies/([^/]*)/series/([^/]*)/rendered", true);
-        OrthancPlugins::RegisterRestCallback<RetrieveInstanceRendered>(root + "studies/([^/]*)/series/([^/]*)/instances/([^/]*)/rendered", true);
-        OrthancPlugins::RegisterRestCallback<RetrieveFrameRendered>(root + "studies/([^/]*)/series/([^/]*)/instances/([^/]*)/frames/([^/]*)/rendered", true);
+        OrthancPlugins::RegisterRestCallback<RetrieveStudyRenderedThumbnail<false> >(root + "studies/([^/]*)/rendered", true);
+        OrthancPlugins::RegisterRestCallback<RetrieveSeriesRenderedThumbnail<false> >(root + "studies/([^/]*)/series/([^/]*)/rendered", true);
+        OrthancPlugins::RegisterRestCallback<RetrieveInstanceRenderedThumbnail<false> >(root + "studies/([^/]*)/series/([^/]*)/instances/([^/]*)/rendered", true);
+        OrthancPlugins::RegisterRestCallback<RetrieveFrameRenderedThumbnail<false> >(root + "studies/([^/]*)/series/([^/]*)/instances/([^/]*)/frames/([^/]*)/rendered", true);
+
+        OrthancPlugins::RegisterRestCallback<RetrieveStudyRenderedThumbnail<true> >(root + "studies/([^/]*)/thumbnail", true);
+        OrthancPlugins::RegisterRestCallback<RetrieveSeriesRenderedThumbnail<true> >(root + "studies/([^/]*)/series/([^/]*)/thumbnail", true);
+        OrthancPlugins::RegisterRestCallback<RetrieveInstanceRenderedThumbnail<true> >(root + "studies/([^/]*)/series/([^/]*)/instances/([^/]*)/thumbnail", true);
+        OrthancPlugins::RegisterRestCallback<RetrieveFrameRenderedThumbnail<true> >(root + "studies/([^/]*)/series/([^/]*)/instances/([^/]*)/frames/([^/]*)/thumbnail", true);
 
         if (!OrthancPlugins::Configuration::IsReadOnly())
         {
