@@ -3,8 +3,8 @@
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
  * Copyright (C) 2017-2023 Osimis S.A., Belgium
- * Copyright (C) 2024-2024 Orthanc Team SRL, Belgium
- * Copyright (C) 2021-2024 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
+ * Copyright (C) 2024-2025 Orthanc Team SRL, Belgium
+ * Copyright (C) 2021-2025 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Affero General Public License
@@ -29,6 +29,8 @@
 #include <Images/ImageTraits.h>
 #include <Logging.h>
 #include <Toolbox.h>
+#include <EmbeddedResources.h>
+
 
 #if ORTHANC_FRAMEWORK_VERSION_IS_ABOVE(1, 9, 7)
 #  include <SerializationToolbox.h>
@@ -821,10 +823,41 @@ static bool ReadDefaultWindow(RenderingParameters& parameters,
 
 
 static void AnswerFrameRendered(OrthancPluginRestOutput* output,
-                                std::string instanceId,
+                                const std::string& instanceId,
+                                const std::string& transferSyntax,
                                 int frame,
-                                const OrthancPluginHttpRequest* request)
+                                const OrthancPluginHttpRequest* request,
+                                bool isThumbnail)  // for now, for every images except for videos, we consider that /rendered is equivalent to /thumbnail
 {
+  // If the instance is a video, we shall provide the video file itself (MP4, ...)
+  Orthanc::DicomTransferSyntax currentSyntax;
+  if (Orthanc::LookupTransferSyntax(currentSyntax, transferSyntax))
+  {
+    if (currentSyntax >= Orthanc::DicomTransferSyntax_MPEG2MainProfileAtMainLevel && currentSyntax <= Orthanc::DicomTransferSyntax_HEVCMain10ProfileLevel5_1)
+    {
+      if (isThumbnail)
+      {
+        // we are not able to extract a thumbnail from a video -> just return a camera icon
+        std::string videoThumbnail;
+        Orthanc::EmbeddedResources::GetFileResource(videoThumbnail, Orthanc::EmbeddedResources::VIDEO_THUMBNAIL);
+        OrthancPluginAnswerBuffer(OrthancPlugins::GetGlobalContext(), output, videoThumbnail.c_str(), videoThumbnail.size(), "image/jpeg");
+        return;
+      }
+      else
+      {
+        OrthancPlugins::RestApiClient apiClient;
+        apiClient.SetPath(std::string("/instances/") + instanceId + "/frames/0/raw");
+        if (apiClient.Execute())
+        {
+          apiClient.Forward(OrthancPlugins::GetGlobalContext(), output);
+          return;
+        }
+      }
+    }
+  }
+
+  // for other media types, try to generate a single image preview.
+
   static const char* const PHOTOMETRIC_INTERPRETATION = "0028,0004";
   static const char* const PER_FRAME_FUNCTIONAL_GROUPS_SEQUENCE = "5200,9230";
   static const char* const PIXEL_VALUE_TRANSFORMATION_SEQUENCE = "0028,9145";
@@ -967,7 +1000,8 @@ static void AnswerFrameRendered(OrthancPluginRestOutput* output,
 
 static void AnswerFrameRendered(OrthancPluginRestOutput* output,
                                 int frame,
-                                const OrthancPluginHttpRequest* request)
+                                const OrthancPluginHttpRequest* request,
+                                bool isThumbnail)
 {
   if (request->method != OrthancPluginHttpMethod_Get)
   {
@@ -976,9 +1010,11 @@ static void AnswerFrameRendered(OrthancPluginRestOutput* output,
   else
   {
     std::string instanceId, studyInstanceUid, seriesInstanceUid, sopInstanceUid;
-    if (LocateInstance(output, instanceId, studyInstanceUid, seriesInstanceUid, sopInstanceUid, request))
+    std::string transferSyntax;
+
+    if (LocateInstance(output, instanceId, studyInstanceUid, seriesInstanceUid, sopInstanceUid, transferSyntax, request))
     {
-      AnswerFrameRendered(output, instanceId, frame, request);
+      AnswerFrameRendered(output, instanceId, transferSyntax, frame, request, isThumbnail);
     }
     else
     {
@@ -990,30 +1026,31 @@ static void AnswerFrameRendered(OrthancPluginRestOutput* output,
 
 void RetrieveInstanceRendered(OrthancPluginRestOutput* output,
                               const char* url,
-                              const OrthancPluginHttpRequest* request)
+                              const OrthancPluginHttpRequest* request,
+                              bool isThumbnail)
 {
   assert(request->groupsCount == 3);
-  AnswerFrameRendered(output, 1 /* first frame */, request);
+  AnswerFrameRendered(output, 1 /* first frame */, request, isThumbnail);
 }
 
 
 void RetrieveFrameRendered(OrthancPluginRestOutput* output,
                            const char* url,
-                           const OrthancPluginHttpRequest* request)
+                           const OrthancPluginHttpRequest* request,
+                           bool isThumbnail)
 {
   assert(request->groupsCount == 4);
   const char* frame = request->groups[3];
 
-  AnswerFrameRendered(output, boost::lexical_cast<int>(frame), request);
+  AnswerFrameRendered(output, boost::lexical_cast<int>(frame), request, isThumbnail);
 }
 
 
 void RetrieveSeriesRendered(OrthancPluginRestOutput* output,
                             const char* url,
-                            const OrthancPluginHttpRequest* request)
+                            const OrthancPluginHttpRequest* request,
+                            bool isThumbnail)
 {
-  static const char* const INSTANCES = "Instances";
-  
   assert(request->groupsCount == 2);
 
   if (request->method != OrthancPluginHttpMethod_Get)
@@ -1022,35 +1059,11 @@ void RetrieveSeriesRendered(OrthancPluginRestOutput* output,
   }
   else
   {
-    std::string orthancId, studyInstanceUid, seriesInstanceUid;
-    if (LocateSeries(output, orthancId, studyInstanceUid, seriesInstanceUid, request))
+    std::string instanceOrthancId, studyInstanceUid, seriesInstanceUid, transferSyntax;
+    if (LocateOneInstance(output, instanceOrthancId, studyInstanceUid, seriesInstanceUid, transferSyntax, request))
     {
-      Json::Value series;
-      if (OrthancPlugins::RestApiGet(series, "/series/" + orthancId, false) &&
-          series.type() == Json::objectValue &&
-          series.isMember(INSTANCES) &&
-          series[INSTANCES].type() == Json::arrayValue &&
-          series[INSTANCES].size() > 0)
-      {
-        std::set<std::string> ids;
-        for (Json::Value::ArrayIndex i = 0; i < series[INSTANCES].size(); i++)
-        {
-          if (series[INSTANCES][i].type() != Json::stringValue)
-          {
-            throw Orthanc::OrthancException(Orthanc::ErrorCode_InternalError);
-          }
-          else
-          {
-            ids.insert(series[INSTANCES][i].asString());
-          }
-        }
-
-        // Retrieve the first instance in alphanumeric order, in order
-        // to always return the same instance
-        std::string instanceId = *ids.begin();
-        AnswerFrameRendered(output, instanceId, 1 /* first frame */, request);
-        return;  // Success
-      }
+      AnswerFrameRendered(output, instanceOrthancId, transferSyntax, 1 /* first frame */, request, isThumbnail);
+      return;  // Success
     }
 
     throw Orthanc::OrthancException(Orthanc::ErrorCode_InexistentItem, "Inexistent series");
@@ -1060,10 +1073,9 @@ void RetrieveSeriesRendered(OrthancPluginRestOutput* output,
 
 void RetrieveStudyRendered(OrthancPluginRestOutput* output,
                            const char* url,
-                           const OrthancPluginHttpRequest* request)
+                           const OrthancPluginHttpRequest* request,
+                           bool isThumbnail)
 {
-  static const char* const ID = "ID";
-  
   assert(request->groupsCount == 1);
 
   if (request->method != OrthancPluginHttpMethod_Get)
@@ -1072,35 +1084,11 @@ void RetrieveStudyRendered(OrthancPluginRestOutput* output,
   }
   else
   {
-    std::string orthancId, studyInstanceUid;
-    if (LocateStudy(output, orthancId, studyInstanceUid, request))
+    std::string instanceOrthancId, studyInstanceUid, seriesInstanceUid, transferSyntax;
+    if (LocateOneInstance(output, instanceOrthancId, studyInstanceUid, seriesInstanceUid, transferSyntax, request))
     {
-      Json::Value instances;
-      if (OrthancPlugins::RestApiGet(instances, "/studies/" + orthancId + "/instances", false) &&
-          instances.type() == Json::arrayValue &&
-          instances.size() > 0)
-      {
-        std::set<std::string> ids;
-        for (Json::Value::ArrayIndex i = 0; i < instances.size(); i++)
-        {
-          if (instances[i].type() != Json::objectValue ||
-              !instances[i].isMember(ID) ||
-              instances[i][ID].type() != Json::stringValue)
-          {
-            throw Orthanc::OrthancException(Orthanc::ErrorCode_InternalError);
-          }
-          else
-          {
-            ids.insert(instances[i][ID].asString());
-          }
-        }
-
-        // Retrieve the first instance in alphanumeric order, in order
-        // to always return the same instance
-        std::string instanceId = *ids.begin();
-        AnswerFrameRendered(output, instanceId, 1 /* first frame */, request);
-        return;  // Success
-      }
+      AnswerFrameRendered(output, instanceOrthancId, transferSyntax, 1 /* first frame */, request, isThumbnail);
+      return;  // Success
     }
 
     throw Orthanc::OrthancException(Orthanc::ErrorCode_InexistentItem, "Inexistent study");
